@@ -13,16 +13,26 @@ impl Grid {
     fn parse(input: &str) -> Result<Self, String> {
         let rows = input.lines().count();
         let cols = input.lines().next().ok_or("Empty input")?.len();
-        let mut locations = vec![(0, 0); 8];
+        let mut locations = Vec::new();
         let mut data = vec![true; rows * cols];
 
         for (y, line) in input.lines().enumerate() {
+            if line.len() != cols {
+                return Err("Not all rows have equal length".into());
+            }
             for (x, &c) in line.as_bytes().iter().enumerate() {
                 data[y * cols + x] = match c {
                     b'#' => false,
                     b'.' => true,
                     b'0'..=b'7' => {
-                        locations[(c - b'0') as usize] = (x, y);
+                        if x == 0 || y == 0 || x + 1 == cols || y + 1 == rows {
+                            return Err("Number at edge".into());
+                        }
+                        let number = (c - b'0') as usize;
+                        if number >= locations.len() {
+                            locations.resize(number + 1, (0, 0));
+                        }
+                        locations[number] = (x, y);
                         true
                     }
                     _ => {
@@ -48,14 +58,17 @@ pub fn solve(input: &mut Input) -> Result<usize, String> {
     let grid = Grid::parse(input.text)?;
     let mut distances: HashMap<(usize, usize), usize> = HashMap::new();
 
-    for from in 0..8 {
-        'toloop: for to in 0..8 {
+    for from in 0..grid.locations.len() {
+        'toloop: for to in 0..grid.locations.len() {
             if to <= from {
                 continue;
             }
 
             let starting_location = grid.locations[from];
             let target_location = grid.locations[to];
+            if starting_location == (0, 0) || target_location == (0, 0) {
+                return Err("Not all digits in grid".into());
+            }
 
             let mut visited = HashSet::new();
             let mut to_visit = BinaryHeap::new();
@@ -91,7 +104,7 @@ pub fn solve(input: &mut Input) -> Result<usize, String> {
         }
     }
 
-    let mut initial_order = [1_usize, 2, 3, 4, 5, 6, 7];
+    let mut initial_order = (1..grid.locations.len()).collect::<Vec<usize>>();
     let mut answer = usize::MAX;
     all_permutations(
         &mut initial_order,
@@ -121,6 +134,8 @@ pub fn solve(input: &mut Input) -> Result<usize, String> {
 pub fn tests() {
     use crate::{test_part_one, test_part_two};
 
+    test_part_one!("###########\n#0.1.....2#\n#.#######.#\n#4.......3#\n###########" => 14);
+
     let real_input = include_str!("day24_input.txt");
     test_part_one!(real_input => 412);
     test_part_two!(real_input => 664);
